@@ -140,18 +140,16 @@ func NewCmdClusterCreate() *cobra.Command {
 				simpleCfg.Name = args[0]
 			}
 
+			if err := config.ProcessSimpleConfig(&simpleCfg); err != nil {
+				l.Log().Fatalf("error processing/sanitizing simple config: %v", err)
+			}
+
 			clusterConfig, err := config.TransformSimpleToClusterConfig(cmd.Context(), runtimes.SelectedRuntime, simpleCfg)
 			if err != nil {
 				l.Log().Fatalln(err)
 			}
 			l.Log().Debugf("===== Merged Cluster Config =====\n%+v\n===== ===== =====\n", clusterConfig)
 
-			clusterConfig, err = config.ProcessClusterConfig(*clusterConfig)
-			if err != nil {
-				l.Log().Fatalln(err)
-			}
-			l.Log().Debugf("===== Processed Cluster Config =====\n%+v\n===== ===== =====\n", clusterConfig)
-
 			if err := config.ValidateClusterConfig(cmd.Context(), runtimes.SelectedRuntime, *clusterConfig); err != nil {
 				l.Log().Fatalln("Failed Cluster Configuration Validation: ", err)
 			}
@@ -291,7 +289,7 @@ func NewCmdClusterCreate() *cobra.Command {
 
 	cmd.Flags().StringP("image", "i", "", "Specify k3s image that you want to use for the nodes")
 	_ = cfgViper.BindPFlag("image", cmd.Flags().Lookup("image"))
-	cfgViper.SetDefault("image", fmt.Sprintf("%s:%s", k3d.DefaultK3sImageRepo, version.GetK3sVersion(false)))
+	cfgViper.SetDefault("image", fmt.Sprintf("%s:%s", k3d.DefaultK3sImageRepo, version.K3sVersion))
 
 	cmd.Flags().String("network", "", "Join an existing network")
 	_ = cfgViper.BindPFlag("network", cmd.Flags().Lookup("network"))
