@@ -25,7 +25,7 @@ import (
 	"bytes"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"os"
 	"strings"
 
 	"sigs.k8s.io/yaml"
@@ -39,7 +39,7 @@ import (
 func ValidateSchemaFile(filepath string, schema []byte) error {
 	l.Log().Debugf("Validating file %s against default JSONSchema...", filepath)
 
-	fileContents, err := ioutil.ReadFile(filepath)
+	fileContents, err := os.ReadFile(filepath)
 	if err != nil {
 		return fmt.Errorf("Failed to read file %s: %+v", filepath, err)
 	}
@@ -53,7 +53,7 @@ func ValidateSchemaFile(filepath string, schema []byte) error {
 }
 
 // ValidateSchema validates a YAML construct (non-struct representation) against a JSON Schema
-func ValidateSchema(content map[string]interface{}, schemaJSON []byte) error {
+func ValidateSchema(content interface{}, schemaJSON []byte) error {
 
 	contentYaml, err := yaml.Marshal(content)
 	if err != nil {
@@ -64,6 +64,11 @@ func ValidateSchema(content map[string]interface{}, schemaJSON []byte) error {
 		return err
 	}
 
+	return ValidateSchemaJSON(contentJSON, schemaJSON)
+
+}
+
+func ValidateSchemaJSON(contentJSON []byte, schemaJSON []byte) error {
 	if bytes.Equal(contentJSON, []byte("null")) {
 		contentJSON = []byte("{}") // non-json yaml struct
 	}
