@@ -22,7 +22,13 @@ THE SOFTWARE.
 
 package util
 
-import "strings"
+import (
+	"bytes"
+	"io"
+	"strings"
+
+	"gopkg.in/yaml.v2"
+)
 
 func RemoveElementFromStringSlice(slice []string, index int) []string {
 	slice[index] = slice[len(slice)-1]
@@ -35,3 +41,26 @@ func ReplaceInAllElements(replacer *strings.Replacer, arr []string) []string {
 	}
 	return arr
 }
+
+func SplitYAML(resources []byte) ([][]byte, error) {
+
+	dec := yaml.NewDecoder(bytes.NewReader(resources))
+
+	var res [][]byte
+	for {
+		var value interface{}
+		err := dec.Decode(&value)
+		if err == io.EOF {
+			break
+		}
+		if err != nil {
+			return nil, err
+		}
+		valueBytes, err := yaml.Marshal(value)
+		if err != nil {
+			return nil, err
+		}
+		res = append(res, valueBytes)
+	}
+	return res, nil
+}
